from dataclasses import MISSING, dataclass, field, fields
from typing import Any, Dict, Union

import random

from mag_tools.format.text_formatter import TextFormatter
from mag_tools.model.justify_type import JustifyType


@dataclass
class BaseData:
    """
    Bean的基类
    """
    _formatter = TextFormatter(
        number_per_line=None,
        justify_type=JustifyType.LEFT,
        at_header='',
        at_end='',
        decimal_places=6,
        decimal_places_of_zero=1,
        pad_length=None,
        pad_char=' ',
        scientific=False,
        none_default='NA'
    )

    def __repr__(self):
        field_dict = {k: v for k, v in self.__dict__.items() if not k.startswith('_')}
        field_str = ', '.join(f"{k}={v}" for k, v in field_dict.items())
        return f"{self.__class__.__name__}({field_str})"

    def __str__(self):
        excluded_attributes = {}
        attributes = [
            f"{attr}={repr(getattr(self, attr))}"
            for attr in vars(self)
            if getattr(self, attr) is not None and attr not in excluded_attributes
        ]
        return f"{self.__class__.__name__}({', '.join(attributes)})"

    @classmethod
    def get_default_value(cls, field_name: str) -> Any:
        """
        获取字段的缺省值
        """
        for f in fields(cls):   # type: ignore
            if f.name == field_name:
                return f.default if f.default is not MISSING else None
        raise ValueError(f"Field '{field_name}' not found in {cls.__name__}")

    @classmethod
    def get_metadata(cls, field_name: str, prop_name: str) -> Any:
        """
        获取字段的metadata属性
        """
        for f in fields(cls):  # type: ignore
            if f.name == field_name:
                return f.metadata.get(prop_name) if f.metadata is not MISSING else None
        raise ValueError(f"Field '{field_name}.{prop_name}' not found in {cls.__name__}")

    @property
    def to_map(self) -> Dict[str, Any]:
        return {
            k: repr(v) for k, v in self.__dict__.items()
            if not (k.startswith('__') and k.endswith('__')) and k not in {'_text_format', '_data_formats'}
        }

    def set_random(self, field_name: str, max_value: Union[float,int] = None, min_value: Union[float,int] = 0) -> Any:
        """
        为指定字段随机生成值并设置到实例属性中。

        :param field_name: 数据类中的字段名
        :param max_value: 最大值（可选）
        :param min_value: 最小值（默认为0）
        """
        for f in fields(self):  # type: ignore
            if f.name == field_name and issubclass(f.type, (int, float)):
                min_ = min_value if min_value else f.metadata.get('min', min_value) if f.metadata is not MISSING else min_value
                max_ = max_value if max_value else f.metadata.get('max', max_value) if f.metadata is not MISSING else max_value

                if min_ is not None and max_ is not None and min_ < max_:
                    value = random.randint(min_, max_) if f.type == int else random.uniform(min_, max_)
                    setattr(self, field_name, value)
                    return value
        raise ValueError(f"Field '{field_name}' not found in {self.__name__}")


import unittest

from mag_tools.utils.data.value_utils import ValueUtils


class TestValueUtils(unittest.TestCase):
    def test_to_value(self):
        print(ValueUtils.to_value('12.50', float))

        # 测试整数转换
        self.assertEqual(ValueUtils.to_value("123", int), 123)
        # 测试浮点数转换
        self.assertEqual(ValueUtils.to_value("123.45", float), 123.45)
        self.assertEqual(ValueUtils.to_value("1.12000E+03", float), 1120)
        # 测试布尔值转换
        self.assertTrue(ValueUtils.to_value("true", bool))
        self.assertFalse(ValueUtils.to_value("false", bool))
        # 测试列表转换
        self.assertEqual(ValueUtils.to_value("[1, 2, 3]", list), [1, 2, 3])
        # 测试字典转换
        self.assertEqual(ValueUtils.to_value("{'key': 'value'}", dict), {'key': 'value'})
        # 测试默认类型转换
        self.assertEqual(ValueUtils.to_value("text"), "text")

    def test_to_values(self):
        str_ = '1.00000E+01 3.00000E+02 7.00000E+02 1.00000E+03 2.00000E+03'
        print(ValueUtils.to_values(str_, float))

        str_ = '1 1 1 1 1.93199E+03 1.36585E+03 6.77031E+02 7.15261E+02 8.62436E+02'
        print(ValueUtils.to_values(str_, float))

        str_ = '12, 23,123,321,12'
        print(ValueUtils.to_values(str_, int, ','))

    def test_number_to_scientific(self):
        print(ValueUtils.to_scientific(123.50, 3, 2))

        # 测试0
        self.assertEqual("0", ValueUtils.to_scientific(0, 5, 2))
        self.assertEqual("0.000", ValueUtils.to_scientific(0.0, 5,3))

        # 测试默认小数位数
        self.assertEqual("1.000E+9", ValueUtils.to_scientific(1000000000.0, decimal_places=5, decimal_places_of_zero=3))
        self.assertEqual("1.230000E-4", ValueUtils.to_scientific(0.000123))

        # 测试指定小数位数
        self.assertEqual("1.00E+9", ValueUtils.to_scientific(1000000000.0, 5, 2))
        self.assertEqual("1.230E-4", ValueUtils.to_scientific(0.000123, 3))

        # 测试负数
        self.assertEqual("-1.0E+9", ValueUtils.to_scientific(-1000000000.0))
        self.assertEqual("-1.230000E-4", ValueUtils.to_scientific(-0.000123))

        # 测试小数位数不足时补零
        self.assertEqual("1.23000E+0", ValueUtils.to_scientific(1.23, 5))
        self.assertEqual("1.00000E+0", ValueUtils.to_scientific(1.0, 5, 5))

    def test_to_string(self):
        print(ValueUtils.to_string(None, 6, 3, False, 'NA'))
        print(ValueUtils.to_string(125000, 6, 3))

        # 测试浮点数格式化
        self.assertEqual(ValueUtils.to_string(123.456, 2), "123.46")
        self.assertEqual("9110", ValueUtils.to_string(9110.0, 5, 0))
        # 测试科学计数法格式化
        self.assertEqual("1.230000E+3", ValueUtils.to_string(1.23e3, 6, 1,True))
        # 测试整数格式化
        self.assertEqual(ValueUtils.to_string(123), "123")

    def test_to_chinese_number(self):
        self.assertEqual(ValueUtils.to_chinese_number(0), "零")
        self.assertEqual(ValueUtils.to_chinese_number(10), "十")
        self.assertEqual(ValueUtils.to_chinese_number(110), "一百一十")
        self.assertEqual(ValueUtils.to_chinese_number(1234), "一千二百三十四")
        self.assertEqual(ValueUtils.to_chinese_number(10001), "一万零一")


from mag_tools.model.base_enum import BaseEnum


class UserType(BaseEnum):
    ROOT = ("Root", "超级管理员")
    CLOUD = ("Cloud", "云平台用户")
    TENANT = ("Tenant", "租户用户")
    ORG = ("Org", "机构用户")
    PERSON = ("Person", "个人用户")
    APPLICATION = ("Application", "应用系统用户")



import struct
from dataclasses import dataclass, field
from typing import Optional, Union

@dataclass
class ArrayHeader:
    array_type: str = field(metadata={'description': '数据类型，i: 整型；d：双精度'})
    array_name: str = field(metadata={'description': '数组名'})
    unit_name: str = field(metadata={'description': '单位名'})
    max_value: Optional[Union[float, int]] = field(default=None, metadata={'description': '最大值'})
    min_value: Optional[Union[float, int]] = field(default=None, metadata={'description': '最小值'})
    mean_value: Optional[Union[float, int]] = field(default=None, metadata={'description': '均值'})
    time_type: Optional[str] = field(default='d', metadata={'description': '时间类型，d: “YYYYMMDD.x”，t: 浮点时间'})
    dt: Optional[float] = field(default=None, metadata={'description': '日期或时间：类型为d时,整数部分为8位日期，小数部分为0.x天; 为i时，为浮点数时间'})
    reserve: Optional[str] = field(default=None, metadata={'description': "保留"})

    @staticmethod
    def from_bytes(header_bytes):
        dt,array_type,array_name,unit_name,max_value,min_value,mean_value,time_type = struct.unpack(
            '=d1s128s64sddd1s39x', header_bytes)

        return ArrayHeader(dt=dt, array_type=array_type.decode('utf-8'),
                           array_name=array_name.decode('utf-8').strip('\x00'),
                           unit_name=unit_name.decode('utf-8').strip('\x00'),
                           max_value=max_value, min_value=min_value,
                           mean_value=mean_value, time_type=time_type.decode('utf-8'))

    def to_bytes(self):
        return struct.pack(
            '=dB128s64sdddB39x',
            self.dt,
            ord(self.array_type),
            self.array_name.encode('utf-8'),
            self.unit_name.encode('utf-8'),
            self.max_value,
            self.min_value,
            self.mean_value,
            ord(self.time_type)
        )

@dataclass
class Array:
    __header: ArrayHeader = field(metadata={'description': '数组头描述'})
    data: list[Union[float, int]] = field(default_factory=list, metadata={'description': '数据，数组元素为整数或浮点数'})

    @property
    def array_name(self) -> str:
        return self.__header.array_name

    @property
    def array_type(self) -> str:
        return self.__header.array_type

    @property
    def unit_name(self) -> str:
        return self.__header.unit_name

    @property
    def max_value(self) -> Union[float, int]:
        return self.__header.max_value

    @property
    def min_value(self) -> Union[float, int]:
        return self.__header.min_value

    @property
    def mean_value(self) -> Union[float, int]:
        return self.__header.mean_value

    @property
    def time_type(self):
        return self.__header.time_type

    @property
    def time_step(self):
        return self.__header.dt

    @classmethod
    def from_bytes(cls, header: ArrayHeader, array_type: str, array_bytes: bytes):
        data = []
        if array_type == 'd':  # 如果类型为 'd'，则读取 double 数组
            array_format = 'd' * (len(array_bytes) // 8)
            data = struct.unpack(array_format, array_bytes)
        elif array_type == 'i':  # 否则读取 int 数组
            array_format = 'i' * (len(array_bytes) // 4)
            data = struct.unpack(array_format, array_bytes)

        return cls(header, data)

    def to_bytes(self) -> bytes:
        #
        """
        将数组转换为字节数组，包括：数组长度、头信息、数组信息
        :return: byte[]
        """
        # 将头信息转为字节数组
        header_bytes = self.__header.to_bytes()

        # 将数据转换为字节数组
        if self.array_type == 'd':
            array_format = 'd' * len(self.data)
        elif self.array_type == 'i':
            array_format = 'i' * len(self.data)
        else:
            raise ValueError("Unsupported array type")
        array_bytes = struct.pack(array_format, *self.data)

        # 将头信息和数组数据总长度转为字节数组
        total_len = len(array_bytes) + len(header_bytes)
        total_len_bytes = struct.pack('q', total_len)

        return total_len_bytes + header_bytes + array_bytes

    @classmethod
    def load_bin_file(cls, bin_file_path: str) -> list:
        """
        bin_file_path 二进制结果文件
            BIN文件中，多个数组连续排列，所有数组都按同样的二进制格式存储。
            每个数组的第 1 至第 8 字节存储数组长度“len”,不含这8个字节；
            第 8 至第 273 字节存储数组的 header 信息；
            第 274 至第(len+8)字节存储数组本身。
            其中，header信息包括：
                8字节的时间或日期（double格式）
                1字节的数组类型标识
                128字节的数组名
                64字节的的单位名
                8字节的最大值(double)
                8字节的最小值（double）
                8字节的均值
                1字节的时间类型
                39字节保留
        :return: Array[]
        """
        data = []
        with open(bin_file_path, 'rb') as f:
            while True:
                # 读取数组字节长度，包含Header和数组本身的byte数
                len_bytes = f.read(8)
                if not len_bytes:
                    break
                array_len = struct.unpack('Q', len_bytes)[0] - 265

                # 读取 header 信息
                header_bytes = f.read(265)

                # 读取数组本身
                array_bytes = f.read(array_len)

                _header = ArrayHeader.from_bytes(header_bytes)
                _array = cls.from_bytes(_header, _header.array_type, array_bytes)
                data.append(_array)
        return data


from mag_tools.model.base_enum import BaseEnum


class MaskingAlg(BaseEnum):
    # 加密
    DES = ("DES", "DES加密")
    DES_EDE = ("DES_EDE", "DesEde加密")
    TRIPLE_DES = ("TRIPLE_DES", "3Des加密")
    AES = ("AES", "AES加密")
    # Hash
    MD_5 = ("MD5", "MD5 Hash")
    SHA_1 = ("SHA1", "SHA1 Hash")
    SHA_256 = ("SHA256", "SHA256 Hash")
    # 掩码
    STAR_REPLACE = ("STAR_REPLACE", "星号替代掩码法")
    RANDOM_REPLACE = ("RANDOM_REPLACE", "随机值替代掩码法")

    def is_masking(self):
        """
        判断是否为掩码算法
        :return: 是否为掩码算法
        """
        return self in {MaskingAlg.STAR_REPLACE, MaskingAlg.RANDOM_REPLACE}

    def is_crypt(self):
        """
        判断是否为加密算法
        :return: 是否为加密算法
        """
        return self in {MaskingAlg.DES, MaskingAlg.DES_EDE, MaskingAlg.TRIPLE_DES, MaskingAlg.AES}

    def is_hash(self):
        """
        判断是否为Hash算法
        :return: 是否为Hash算法
        """
        return self in {MaskingAlg.MD_5, MaskingAlg.SHA_1, MaskingAlg.SHA_256}


if __name__ == "__main__":
    # 示例使用
    print(MaskingAlg.of_code("AES"))  # 输出: MaskingAlg.AES
    print(MaskingAlg.AES.is_crypt())  # 输出: True
    print(MaskingAlg.MD_5.is_hash())  # 输出: True
    print(MaskingAlg.STAR_REPLACE.is_masking())  # 输出: True



from mag_tools.model.base_enum import BaseEnum

class TimeUnit(BaseEnum):
    NANOSECOND = ('NANOSECOND', '纳秒')
    MICROSECOND = ('MICROSECOND', '微秒')
    MILLISECOND = ('MILLISECOND', '毫秒')
    SECOND = ('SECOND', '秒')
    MINUTE = ('MINUTE', '分')
    HOUR = ('HOUR', '时')
    DAY = ('DAY', '天')
    WEEK = ('WEEK', '周')
    MONTH = ('MONTH', '月')
    YEAR = ('YEAR', '年')



import re
from datetime import date, datetime, time
from typing import Any, List, Optional, Union

import numpy as np


class ValueUtils:
    @staticmethod
    def to_value(text: str, data_type: Optional[type] = None) -> Optional[Any]:
        """
        将文本转换为数值
        :param text: 文本
        :param data_type: 数据类型
        """
        if text is None or not isinstance(text, str):
            return None

        text = text.strip()
        if data_type == int:
            try:
                return int(text)
            except ValueError:
                return None
        elif data_type == float:
            try:
                if text.endswith('%'):
                    return float(text[:-1]) / 100

                return float(text)
            except ValueError:
                return None
        elif data_type == bool:
            text = text.lower()
            return text in ['true', 'yes', 't', 'y', '1']
        elif data_type == list:
            return eval(text)
        elif data_type == dict:
            return eval(text)
        else:
            return text

    @staticmethod
    def to_values(text: str, data_type: Optional[type] = None, sep: str = ' ') -> List[Any]:
        if text is None or not isinstance(text, str):
            return []

        text = text.strip()
        items = text.split(sep)
        return [ValueUtils.to_value(item, data_type) for item in items]

    @staticmethod
    def to_string(value: Optional[Union[int, float, bool, str, datetime, date, time]],
                  decimal_places: int=6,
                  decimal_places_of_zero: int=1,
                  scientific: bool = False,
                  none_default: str = None) -> str:
        """
        将给定的值转换为字符串表示
        :param value: 要转换的值，支持 int, float, bool, 和 str 类型
        :param decimal_places: 指定浮点数保留的最大小数位数，默认为 6
        :param decimal_places_of_zero: 当浮点数的值为整数或接近整数时，最少保留的小数位数，默认为 1
        :param scientific: 是否强制使用科学记数法格式，默认为 False
        :param none_default: 当数据(int或float)为None时对应的缺省字符串
        :return: 转换为字符串的值，格式根据参数动态调整
        详细说明:
        1. 整数、布尔值、字符串类型: 直接转换为字符串。

        2. 根据数值范围决定是否使用科学记数法:
            - 如果值的绝对值大于等于 1e8 或 (不为零并且绝对值小于 1e-3)，默认使用科学记数法。
            - 整数类型始终不使用科学记数法，除非显式指定使用科学计数法

        3. 浮点数的常规格式:
            - 如果浮点数为整数（即小数部分全为零），按 `decimal_places_of_zero` 指定的小数位数格式化。
            - 如果浮点数有小数部分，则按 `decimal_places` 指定的小数位数格式化。
            - 格式化后的小数部分会移除多余的零，但确保至少保留 `decimal_places_of_zero` 个小数位。
        """
        if value is None:
            return none_default

        if isinstance(value, str) or isinstance(value, bool):
            return str(value)
        elif isinstance(value, datetime):
            return value.strftime('%Y-%m-%d %H:%M:%S') if value is not None else None
        elif isinstance(value, date):
            return value.strftime('%Y-%m-%d') if value is not None else None
        elif isinstance(value, time):
            return value.strftime('%H:%M:%S') if value is not None else None
        else:
            if abs(value) >= 1e8 or (value != 0 and abs(value) < 1e-3):
                scientific = True
            elif isinstance(value, int):
                scientific = False

            if scientific:
                return ValueUtils.to_scientific(value, decimal_places, decimal_places_of_zero)

            if isinstance(value, float):
                if value.is_integer():
                    return f"{value:.{decimal_places_of_zero}f}"
                else:
                    formatted_value = f"{value:.{decimal_places}f}"
                    # 确保小数位末尾的零的个数符合要求
                    if '.' in formatted_value:
                        integer_part, decimal_part = formatted_value.split('.')
                        decimal_part = decimal_part.rstrip('0')
                        if len(decimal_part) < decimal_places_of_zero:
                            decimal_part += '0' * (decimal_places_of_zero - len(decimal_part))
                        formatted_value = f"{integer_part}.{decimal_part}"
                    return formatted_value
            elif isinstance(value, bool):
                return str(value)
            elif isinstance(value, int):
                return f"{round(value)}"

    @staticmethod
    def to_native(value: Any) -> Any:
        """
        将值转换为原生类型，如果可能
        :param value: 原始值
        :return: 转换后的原生类型值
        """
        if isinstance(value, (np.integer, int)):
            return int(value)  # 转换为 Python 的 int 类型
        elif isinstance(value, (np.floating, float)):
            return float(value)  # 转换为 Python 的 float 类型
        elif isinstance(value, (np.bool_, bool)):
            return bool(value)  # 转换为 Python 的 bool 类型
        else:
            return str(value)  # 保留为字符串

    @staticmethod
    def to_scientific(value: Union[int,float], decimal_places: int = 6, decimal_places_of_zero: int = 1) -> str:
        """
        将 float 数字转换为科学计数法表示的字符串。

        参数：
        :param value: float 数字
        :param decimal_places: 小数位数，默认为 6
        :return: 科学计数法表示的字符串
        """
        if not isinstance(value, (int, float)):
            raise TypeError("value 参数必须是 int 或 float 类型。")
        if not isinstance(decimal_places, int) or decimal_places < 0:
            raise ValueError("decimal_places 必须是一个非负整数。")
        if not isinstance(decimal_places_of_zero, int) or decimal_places_of_zero < 0:
            raise ValueError("参数 decimal_places_of_zero 必须是一个非负整数。")

        if value == 0:
            return '0' if isinstance(value, int) else f"0.{'0' * decimal_places_of_zero}"

        exponent = int(f"{value:e}".split('e')[1])  # 指数部分
        coefficient = value / (10 ** exponent)      # 系数部分

        if coefficient == int(coefficient):
            formatted_coefficient = f"{int(coefficient)}.{'0' * decimal_places_of_zero}" if decimal_places_of_zero > 0 else f"{int(coefficient)}"
        else:
            formatted_coefficient = f"{coefficient:.{decimal_places}f}"

        return f"{formatted_coefficient}E{'+' if exponent >= 0 else ''}{exponent}"

    @staticmethod
    def to_chinese_number(num: int) -> str:
        units = ["", "十", "百", "千", "万", "十", "百", "千", "亿"]
        digits = ["零", "一", "二", "三", "四", "五", "六", "七", "八", "九"]

        if num == 0:
            return "零"

        result = ""
        unit_position = 0
        while num > 0:
            digit = num % 10
            if digit != 0:
                result = f"{digits[digit]}{units[unit_position]}{result}"
            elif result and result[0] != "零":
                result = "零" + result
            num //= 10
            unit_position += 1

        # 处理 "一十" 的情况
        if result.startswith("一十"):
            result = result[1:]

        return result

    @staticmethod
    def pick_numbers(text: str) -> List[Union[int, float]]:
        numbers = []

        matches = re.findall(r'[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?', text)
        for match in matches:
            try:
                # 转换为 float，如果是整数则转为 int
                num = float(match)
                if num.is_integer():
                    numbers.append(int(num))
                else:
                    numbers.append(num)
            except ValueError:
                pass
        return numbers

    @staticmethod
    def pick_number(text: str) -> Optional[Union[int, float]]:
        numbers = ValueUtils.pick_numbers(text)
        return numbers[0] if len(numbers) > 0 else None

    @staticmethod
    def get_type(value: Any) -> Optional[type]:
        """
        判断输入值的数据类型

        参数:
        value: 要判断的数据

        返回:
        str: 数据类型
        """
        if isinstance(value, int):
            return int
        elif isinstance(value, float):
            return float
        elif isinstance(value, complex):
            return complex
        elif isinstance(value, bool):
            return bool
        elif isinstance(value, str):
            return str
        else:
            return None


import re
import subprocess
from dataclasses import dataclass, field
from typing import Optional

import wmi

from mag_tools.bean.base_data import BaseData
from mag_tools.bean.sys.operate_system import OperateSystem
from mag_tools.model.memory_type import MemoryType
from mag_tools.utils.security.digest import Digest


@dataclass
class MemoryModule(BaseData):
    sn: Optional[str] = field(default=None, metadata={"description": "序号"})
    computer_id: Optional[str] = field(default=None, metadata={"description": "计算机资源标识"})
    serial_number: Optional[str] = field(default=None, metadata={"description": "序列号"})
    memory_type: Optional[MemoryType] = field(default=None, metadata={"description": "内存类型"})
    capacity: Optional[int] = field(default=None, metadata={"description": "容量 (GB)"})
    speed: Optional[int] = field(default=None, metadata={"description": "频率 (MHz)"})
    voltage: Optional[float] = field(default=None, metadata={"description": "电压 (mV)"})
    form_factor: Optional[str] = field(default=None, metadata={"description": "形态"})
    part_number: Optional[str] = field(default=None, metadata={"description": "零件编号"})
    manufacturer: Optional[str] = field(default=None, metadata={"description": "生产厂商"})
    latency: Optional[int] = field(default=None, metadata={"description": "延迟 (CL)"})
    bank_label: Optional[int] = field(default=None, metadata={"description": "插槽位置标签"})
    device_locator: Optional[int] = field(default=None, metadata={"description": "内存所在位置"})
    data_width: Optional[int] = field(default=None, metadata={"description": "数据宽度，以位为单位"})
    tag: Optional[str] = field(default=None, metadata={"description": "标签"})

    @property
    def hash(self):
        return Digest.md5(f'{self.serial_number}{self.memory_type}{self.capacity}{self.speed}{self.voltage}{self.latency}{self.manufacturer}')

    @classmethod
    def get_info(cls):
        if OperateSystem.is_windows():
            return cls.__get_from_windows()
        else:
            return cls.__get_from_linux()

    def __str__(self):
        """
        返回内存条的字符串表示
        """
        return f"MemoryModule(memory_type='{self.memory_type}', capacity={self.capacity} MB, frequency={self.speed} MHz, voltage={self.voltage} V, latency={self.latency} CL, form_factor='{self.form_factor}', serial_number='{self.serial_number}')"

    @classmethod
    def __get_from_windows(cls):
        modules = []
        c = wmi.WMI()
        for memory in c.Win32_PhysicalMemory():
            modules.append(cls(serial_number=memory.SerialNumber,
                               memory_type=MemoryType.of_code(memory.MemoryType),
                               capacity=int(memory.Capacity) // (1024 ** 3),  # 将字节转换为GB
                               speed=memory.Speed,
                               voltage=memory.ConfiguredVoltage,
                               form_factor=memory.FormFactor,
                               part_number=memory.PartNumber,
                               bank_label=memory.BankLabel,
                               device_locator=memory.DeviceLocator,
                               data_width=memory.DataWidth,
                                tag=memory.Tag
                               ))

        return modules

    @classmethod
    def __get_from_linux(cls):
        """
        获取当前系统的所有内存条信息，并返回一个包含多个Memory实例的列表
        """
        result = subprocess.run(['sudo', 'dmidecode', '--type', 'memory'], stdout=subprocess.PIPE)
        output = result.stdout.decode()

        modules = []
        for section in output.split('\n\n'):
            memory_type = capacity = frequency = voltage = form_factor = serial_number = None

            for line in section.split('\n'):
                if 'Type:' in line and 'Type Detail:' not in line:
                    memory_type = line.split(':')[-1].strip()
                elif 'Size:' in line and 'No Module Installed' not in line:
                    capacity = int(re.findall(r'\d+', line.split(':')[-1].strip())[0])
                elif 'Speed:' in line:
                    frequency = int(re.findall(r'\d+', line.split(':')[-1].strip())[0])
                elif 'Voltage:' in line:
                    voltage = float(re.findall(r'\d+\.\d+', line.split(':')[-1].strip())[0])
                elif 'Form Factor:' in line:
                    form_factor = line.split(':')[-1].strip()
                elif 'Serial Number:' in line:
                    serial_number = line.split(':')[-1].strip()

            if memory_type and capacity:
                modules.append(MemoryModule(memory_type=memory_type, capacity=capacity, speed=frequency, voltage=voltage,
                                            form_factor=form_factor, serial_number=serial_number))

        return modules

# 示例调用
if __name__ == "__main__":
    modules_ = MemoryModule.get_info()
    for module in modules_:
        print(module)



import unittest

from bean.array import Array, ArrayHeader
from mag_tools.format.text_formatter import TextFormatter

from mag_tools.model.justify_type import JustifyType
from utils.data.list_utils import ListUtils
from utils.data.value_utils import ValueUtils


class TestTextFormat(unittest.TestCase):
    def setUp(self):
        self.format = TextFormatter(
            number_per_line=3,
            justify_type=JustifyType.LEFT,
            at_header='   ',
            decimal_places=3,
            decimal_places_of_zero=3,
            pad_length=None,
            pad_char=' ',
            scientific=False,
            none_default='NA'
        )

    def test_array_1d_to_lines(self):
        text_format = TextFormatter(number_per_line=3, pad_length=0, justify_type=JustifyType.LEFT)

        # 测试整数数组
        array_1d = [1, 1, 2, 3, 3, 3]
        expected_output = ['2*1 2   3*3']
        text_format.merge_duplicate = True
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        array_1d = [1.1, 1.1, 2.2, 3.3, 3.3, 3.3]
        expected_output = ['2*1.1 2.2   3*3.3']
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        array_1d = [True, True, False, True, True, True]
        expected_output = ['2*True False  3*True']
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        array_1d = ['a', 'a', 'b', 'c', 'c', 'c']
        expected_output = ['a a b', 'c c c']
        text_format.merge_duplicate = False
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

    def test_array_2d_to_lines(self):
        text_format = TextFormatter(number_per_line=3, pad_length=0, justify_type=JustifyType.LEFT)

        # 测试整数数组
        array_2d = [
            [1, 1, 2],
            [3, 3, 3]
        ]
        expected_output = ['2*1 2   3*3']
        text_format.group_by_layer = False
        text_format.merge_duplicate = True
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        expected_output = ['2*1 2  ','3*3']
        text_format.group_by_layer = True
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        array_2d = [
            [1.1, 1.1, 2.2],
            [3.3, 3.3, 3.3]
        ]
        expected_output = ['2*1.1 2.2   3*3.3']
        text_format.group_by_layer = False
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        expected_output = ['2*1.1 2.2  ', '3*3.3']
        text_format.group_by_layer = True
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        array_2d = [
            [True, True, False],
            [True, True, True]
        ]
        expected_output = ['2*True False  3*True']
        text_format.group_by_layer = False
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        expected_output = ['2*True False ', '3*True']
        text_format.group_by_layer = True
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        array_2d = [
            ['a', 'a', 'b'],
            ['c', 'c', 'c']
        ]
        expected_output = ['a a b', 'c c c']
        text_format.group_by_layer = False
        text_format.merge_duplicate = False
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        expected_output = ['a a b', 'c c c']
        text_format.group_by_layer = True
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

    def test_array_3d_to_lines(self):
        text_format = TextFormatter(number_per_line=4, pad_length=0, justify_type=JustifyType.LEFT)

        # 测试整数数组
        array_3d = [
            [
                [1, 1, 2],
                [2, 3, 3]
            ],
            [
                [4, 4, 5],
                [5, 6, 6]
            ]
        ]
        text_format.group_by_layer = False
        text_format.merge_duplicate = True
        expected_output = ['2*1 2*2 2*3 2*4','2*5 2*6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        text_format.group_by_layer = True
        expected_output = ['2*1 2*2 2*3', '2*4 2*5 2*6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        text_format.merge_duplicate = False
        expected_output = ['1 1 2 2', '3 3', '4 4 5 5', '6 6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        # 测试浮点数数组
        array_3d = [
            [
                [1.1, 1.1, 2.2],
                [3.3, 3.3, 3.3]
            ],
            [
                [4.4, 4.4, 5.5],
                [6.6, 6.6, 6.6]
            ]
        ]
        text_format.merge_duplicate = True
        text_format.group_by_layer = False
        expected_output = ['2*1.1 2.2   3*3.3 2*4.4','5.5   3*6.6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        text_format.group_by_layer = True
        expected_output = ['2*1.1 2.2   3*3.3', '2*4.4 5.5   3*6.6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        # 测试布尔值数组
        array_3d = [
            [
                [True, True, False],
                [True, True, True]
            ],
            [
                [False, False, True],
                [False, False, False]
            ]
        ]
        expected_output = ['2*True  False   3*True  2*False', 'True    3*False']
        text_format.group_by_layer = False
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        expected_output = ['2*True  False   3*True ', '2*False True    3*False']
        text_format.group_by_layer = True
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        # 测试字符串数组
        array_3d = [
            [
                ['a', 'a', 'b'],
                ['c', 'c', 'c']
            ],
            [
                ['d', 'd', 'e'],
                ['f', 'f', 'f']
            ]
        ]
        expected_output = ['a a b c', 'c c d d', 'e f f f']
        text_format.group_by_layer = False
        text_format.merge_duplicate = False
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        expected_output = ['a a b c', 'c c', 'd d e f', 'f f']
        text_format.group_by_layer = True
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

    def test_arrays_to_lines(self):
        str_ = '''
XCOORD       ft          7050          7050          7050          7050          7050          7050
YCOORD       ft          7350          7350          7350          7350          7350          7350
 DEPTH       ft          9110      10378.16      10396.66      10415.66      10450.66      10525.66
 STAGE  integer             0             0             0             0             0             0
OUTLET  integer            -1             0             1             2             3             4    
            '''

        arrays = []
        # 解析数据块，每行为一个数组，由若干数组组成。格式：数组名、单位和数据
        block_lines = ListUtils.trim(str_.splitlines())
        for line in block_lines:
            items = line.split()
            if items[0] in {'XCOORD', 'YCOORD', 'DEPTH'}:
                array_type = 'd'
            else:
                array_type = 'i'

            head = ArrayHeader(array_type=array_type, array_name=items[0], unit_name=items[1])
            data = [ValueUtils.to_value(value, float) if array_type == 'd' else int(value) for value in items[2:]]
            arrays.append(Array(head, data))

        lines = TextFormatter().arrays_to_lines(arrays)
        print('\n'.join(lines))


from enum import Enum
from typing import Optional, Union


class BaseEnum(Enum):
    def __init__(self, code: Union[str, int], desc: Optional[str] = None):
        self._code = code
        self._desc = desc

    @classmethod
    def of_name(cls, name: str, default_value=None) :
        return getattr(cls, name, default_value)

    @classmethod
    def of_code(cls, code: Optional[Union[str, int]], default_value=None):
        """
        根据代码获取枚举
        :param code: 代码
        :param default_value: 缺省值
        :return: 枚举
        """
        if code is not None:
            for _enum in cls:
                if str(_enum.code).upper() == str(code).upper():
                    return _enum
        return default_value

    @classmethod
    def of_desc(cls, desc: str, default_value=None):
        for _enum in cls:
            if _enum.desc == desc:
                return _enum
        return default_value

    @property
    def code(self):
        return self._code

    @property
    def desc(self):
        return self._desc

    @classmethod
    def codes(cls):
        return [enum_member.code for enum_member in cls]

    def __str__(self):
        return f"{self.name}[code={self.code}, desc={self.desc}]"




from mag_tools.model.base_enum import BaseEnum


class CpuSeries(BaseEnum):
    INTEL_I3 = ("I3", "Intel Core i3")
    INTEL_I5 = ("I5", "Intel Core i5")
    INTEL_I7 = ("I7", "Intel Core i7")
    INTEL_I9 = ("I9", "Intel Core i9")
    AMD_3 = ("R3", "AMD Ryzen 3")
    AMD_5 = ("R5", "AMD Ryzen 5")
    AMD_7 = ("R7", "AMD Ryzen 7")
    AMD_9 = ("R9", "AMD Ryzen 9")